package com.gcotu_ootp.obsrecordingdetector;

import java.awt.*;
import java.io.File;
import java.util.Arrays;




final public class ArgUtils {


    final public static String PROGRAM_NAMEANDVERSION = "OBS Recording Detector v1 (2024-06-02)";
    final public static String PROGRAM_NAMEANDVERSION_ANDMAKER = PROGRAM_NAMEANDVERSION + " by God, creator of the universe (YouTube; OwnerOfThePlace on Discord)";

    final private static int INTERVAL_DEFAULT = 200;
    final private static int INTERVAL_MIN = 10;
    final private static int INTERVAL_MAX = 1000 * 60; // 60 seconds (RIDICULOUS!)

    final private static int WINSIZE_DEFAULT_X = 70;
    final private static int WINSIZE_DEFAULT_Y = 70;
    final private static int WINSIZE_MIN_X = 10;
    final private static int WINSIZE_MIN_Y = 10;
    final private static int WINSIZE_MAX_X = 1000;
    final private static int WINSIZE_MAX_Y = 1000;

    // THESE STRINGS must stay LOWER CASE!
    final private static String ARG_FOLDER = "folder="; // Where OBS records the videos.
    final private static String ARG_INTERVAL = "interval="; // What folder check rhythm you want.
    final private static String ARG_WIN_SIZE = "win_size=";
    final private static String ARG_WIN_NOTBORDERLESS = "win_notborderless";
    final private static String ARG_WIN_NOTFRONTMOST = "win_notfrontmost";
    final private static String ARG_DEMO = "demo";
    final private static String ARG_DEBUG = "debug"; // Adds superfluous printout.


    public static boolean checkIfDebugMode(final String[] args) {

        return checkIfArgExists(args, ARG_DEBUG, "-" + ARG_DEBUG, "debugmode", "-debugmode");
    }


    public static boolean checkIfDemoMode(final String[] args) {

        return checkIfArgExists(args, ARG_DEMO, "-" + ARG_DEMO, "demomode", "-demomode");
    }


    public static void maybePrintHelpAndQuit(final String[] args,
                                             final ImageHandler imageHandler) {

        if (args == null || args.length == 0 || checkIfArgExists(args,
                                                                 "?", "-?", "/?",
                                                                 "help", "-help", "/help",
                                                                 "hlp", "-hlp", "/hlp",
                                                                 "doc", "docs",
                                                                 "-doc", "-docs",
                                                                 "/doc", "/docs",
                                                                 "manual", "-manual", "/manual",
                                                                 "man", "-man", "/man")) {

            System.err.println();
            System.err.println(PROGRAM_NAMEANDVERSION_ANDMAKER);
            final char[] underline = new char[PROGRAM_NAMEANDVERSION_ANDMAKER.length()];
            Arrays.fill(underline, '-');
            System.err.println(new String(underline));

            System.err.println();
            System.err.println("CAUTION:");
            System.err.println("- The detector has almost a second of delay.");
            System.err.println(
                    "- This program was developed on Windows 11 using files on local storage. ON A SERVER or different OS, the file system behavior may be different. (This program uses file renaming to detect whether the newest file (Video or not.) is still under lock and key.)");
            System.err.println();
            System.err.println(ARG_FOLDER + " [REQUIRED!] Defines OBS recording folder. Example: \"" + ARG_FOLDER + "C:\\OBSrecordings\" Include these quotation marks!");
            System.err.println();
            System.err.println(ARG_INTERVAL + " [OPTIONAL, default/min/max " +
                                       INTERVAL_DEFAULT + "/" + INTERVAL_MIN + "/" + INTERVAL_MAX +
                                       "] The folder-read & file-check interval. Fast = more load on system (And (UNTESTED!) may actually falsely believe file is not being changed.), slow = information update is delayed.");
            System.err.println();
            System.err.println(ARG_WIN_SIZE + " [OPTIONAL, default/min/max " +
                                       +WINSIZE_DEFAULT_X + "x" + WINSIZE_DEFAULT_Y + "/"
                                       + WINSIZE_MIN_X + "x" + WINSIZE_MIN_Y + "/"
                                       + WINSIZE_MAX_X + "x" + WINSIZE_MAX_Y +
                                       "] Defines the window size.");
            System.err.println();
            System.err.println(ARG_WIN_NOTFRONTMOST + " [OPTIONAL] Window will always be in front, but this flag disables that.");
            System.err.println();
            System.err.println(ARG_WIN_NOTBORDERLESS + " [OPTIONAL] Window will not have a border, but this flag disables that.");
            System.err.println();
            System.err.println(ARG_DEMO + " [OPTIONAL] Disables normal functionality: RIGHT MOUSE BUTTON to toggle the 3 symbols.");
            System.err.println();
            System.err.println(ARG_DEBUG + " [OPTIONAL] Causes printout nobody needs.");
            imageHandler.printImageFileNames();

            System.exit(0);
        }
    }


    public static File getFolderFromArgs(final String[] args) {

        final String arg = getArg(args, ARG_FOLDER);
        if (arg == null) {
            System.err.println("Need folder arg! Use \"help\" or \"?\" for docs.");
            System.exit(-1);
        }

        final File ret = new File(arg);
        if (!ret.isDirectory()) {
            System.err.println("Folder of \"" + ARG_FOLDER + "\" does not exist (may be a relative-path issue): \"" + arg + "\"");
            System.exit(-1);
        }
        return ret;
    }


    public static int getIntervalFromArgs(final String[] args) {

        final String arg = getArg(args, ARG_INTERVAL);
        if (arg == null) {
            return INTERVAL_DEFAULT;
        } else {
            final Integer ret = parseInteger(arg);
            if (ret == null) {
                System.err.println("\"" + ARG_INTERVAL + "\" is not an integer number: \"" + arg + "\"");
                System.exit(-1);
            } else if (ret < INTERVAL_MIN || ret > INTERVAL_MAX) {
                System.err.println("\"" + ARG_INTERVAL + "\" is outside range of " + INTERVAL_MIN + ".." + INTERVAL_MAX + ": \"" + arg + "\"");
                System.exit(-1);
            }
            return ret;
        }
    }


    public static boolean checkIfNotFrontmost(final String[] args) {

        return checkIfArgExists(args, ARG_WIN_NOTFRONTMOST, "-" + ARG_WIN_NOTFRONTMOST);
    }


    public static boolean checkIfNotBorderless(final String[] args) {

        return checkIfArgExists(args, ARG_WIN_NOTBORDERLESS, "-" + ARG_WIN_NOTBORDERLESS);
    }


    public static Dimension getWindowSize(final String[] args) {

        final String arg = getArg(args, ARG_WIN_SIZE);
        if (arg == null) {
            return new Dimension(WINSIZE_DEFAULT_X, WINSIZE_DEFAULT_Y);
        } else {
            final int idxX = arg.indexOf('x');
            if (idxX < 0) {
                System.err.println("\"" + ARG_WIN_SIZE + "\" needs an 'x', e.g. \"200x200\": \"" + arg + "\"");
                System.exit(-1);
            }
            final String wStr = arg.substring(0, idxX).trim();
            final String hStr = arg.substring(idxX + 1).trim();
            final Integer w = parseInteger(wStr);
            final Integer h = parseInteger(hStr);
            if (w == null || h == null) {
                System.err.println("\"" + ARG_WIN_SIZE + "\" is not using integers on both sides of the x (e.g. \"200x200\"): \"" + arg + "\"");
                System.exit(-1);
            }

            if (w < WINSIZE_MIN_X || w > WINSIZE_MAX_X ||
                    h < WINSIZE_MIN_Y || h > WINSIZE_MAX_Y) {
                System.err.println("\"" + ARG_WIN_SIZE + "\" is outside range of " + WINSIZE_MIN_X + "x" + WINSIZE_MIN_Y + ".." + WINSIZE_MAX_X + "x" + WINSIZE_MAX_Y + ": \"" + arg + "\"");
                System.exit(-1);
            }
            return new Dimension(w, h);
        }
    }


    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    private static String getArg(final String[] args,
                                 final String argName) {

        if (args != null) {
            for (String arg : args) {
                if (arg.toLowerCase().startsWith(argName)) {
                    final String ret = arg.substring(argName.length()).trim();
                    return ret.isEmpty() ? null : ret;
                }
            }
        }
        return null;
    }


    private static boolean checkIfArgExists(final String[] args,
                                            final String... argNames) {

        if (args != null) {
            for (String arg : args) {
                for (String n : argNames) {
                    if (arg.equalsIgnoreCase(n)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }


    private static Integer parseInteger(final String arg) {

        try {
            return Integer.parseInt(arg);
        } catch (Exception e) {
            return null;
        }
    }


}
