package com.gcotu_ootp.obsrecordingdetector;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;




final public class Main {


    public static void main(final String[] args) {

        // DEAL WITH ARGUMENTS
        final ImageHandler imageHandler = new ImageHandler(new File("").getAbsoluteFile());
        ArgUtils.maybePrintHelpAndQuit(args, imageHandler);
        final boolean debugMode = ArgUtils.checkIfDebugMode(args);
        final boolean demoMode = ArgUtils.checkIfDemoMode(args);
        final File folder = ArgUtils.getFolderFromArgs(args);
        final int interval = ArgUtils.getIntervalFromArgs(args);
        final boolean win_notFrontmost = ArgUtils.checkIfNotFrontmost(args);
        final boolean win_notBorderless = ArgUtils.checkIfNotBorderless(args);
        final Dimension win_size = ArgUtils.getWindowSize(args);

        SwingUtilities.invokeLater(() -> {
            final Main gui = new Main(imageHandler, demoMode);
            gui.start(win_notFrontmost, win_notBorderless, win_size);
            if (!demoMode) {
                final Thread t = new Thread(() -> THREADCODE_fileStateDetector(folder, interval, debugMode, gui.display::repaint));
                t.setDaemon(false); // Thread MUST have opportunity to rename file BACK!
                t.start();
            }
        });

    }


    private volatile static boolean keepRunning = true;
    private volatile static Boolean recording_nullNo_falsePause_trueYes = null;


    private static void THREADCODE_fileStateDetector(final File folder,
                                                     final int interval,
                                                     final boolean debugMode,
                                                     final Runnable guiUpdater_canBeNull) {

        FileWithDate newestFile_previous = null;

        while (keepRunning) {
            try {
                Thread.sleep(interval);
            } catch (InterruptedException ignored) {
                continue;
            }

            final FileWithDate newestFile = FileWithDate.identifyNewestFile(folder);
            final FileWithDate newestFile_previous_saved = newestFile_previous;
            newestFile_previous = newestFile;

            if (newestFile != null) {
                if (FileWithDate.detectIfOBSHasFileUnderLockAndKey(newestFile)) {
                    if (newestFile.equals(newestFile_previous_saved)) {
                        recording_nullNo_falsePause_trueYes = newestFile.lastModified != newestFile_previous_saved.lastModified;
                    } else {
                        recording_nullNo_falsePause_trueYes = null;
                    }
                } else {
                    recording_nullNo_falsePause_trueYes = null;
                }
            } else {
                recording_nullNo_falsePause_trueYes = null;
            }
            if (debugMode) {
                System.err.println(newestFile + "\t\t" + recording_nullNo_falsePause_trueYes);
            }

            if (guiUpdater_canBeNull != null) {
                SwingUtilities.invokeLater(guiUpdater_canBeNull);
            }

        }

        System.exit(0); // Not needed but also never wrong in this place.

    }


    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    final private JFrame window;
    final private JPanel display;
    final private ImageHandler imageHandler;

    final private boolean demoMode;


    private Main(final ImageHandler imageHandler,
                 final boolean demoMode) {

        this.imageHandler = imageHandler;
        this.demoMode = demoMode;

        display = new JPanel() {

            @Override
            public void paint(final Graphics graphics) {

                // Use this override to BE quick.

                final Graphics2D g = (Graphics2D) graphics;
                final int w = getWidth();
                final int h = getHeight();

                g.setColor(Color.BLACK);
                g.fillRect(0, 0, w, h);

                final BufferedImage image = imageHandler.getImage(recording_nullNo_falsePause_trueYes);
                if (image != null) {
                    final int imgW = image.getWidth();
                    final int imgH = image.getHeight();
                    final int x = (w - imgW) / 2;
                    final int y = (h - imgH) / 2;
                    g.drawImage(image, x, y, null);
                }

                if (demoMode) {
                    final FontMetrics fm = g.getFontMetrics();
                    final String text = "DEMO";
                    g.setColor(Color.GREEN);
                    g.drawString(text,(w-fm.stringWidth(text))/2,(h/2)+ Math.round(fm.getHeight()*0.2f));
                }

            }


        };
        display.setBackground(Color.BLACK);
        final MyMouseAdapter mad = new MyMouseAdapter();
        display.addMouseListener(mad);
        display.addMouseMotionListener(mad);

        window = new JFrame(ArgUtils.PROGRAM_NAMEANDVERSION);
        window.setBackground(Color.BLACK);
        window.setContentPane(display);
        window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        window.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(final WindowEvent e) {

                dispose();
            }
        });
    }


    private void dispose() {

        keepRunning = false;
        window.dispose();
    }


    private void start(final boolean win_notFrontmost,
                       final boolean win_notBorderless,
                       final Dimension win_size) {

        imageHandler.loadImages();

        window.setUndecorated(!win_notBorderless);
        window.setResizable(false);
        window.setSize(win_size);
        window.setAlwaysOnTop(!win_notFrontmost);
        window.setLocationRelativeTo(null);
        window.setVisible(true);
    }


    final private class MyMouseAdapter extends MouseAdapter {


        private Point mouseIsDown = null;


        @Override
        public void mousePressed(final MouseEvent e) {

            final int button = e.getButton();
            if (button == MouseEvent.BUTTON2) {
                dispose();
            } else if (button == MouseEvent.BUTTON1) {
                mouseIsDown = e.getLocationOnScreen();
            } else if (button == MouseEvent.BUTTON3 && demoMode) {
                if (recording_nullNo_falsePause_trueYes==null) {
                    recording_nullNo_falsePause_trueYes= true;
                }else if (recording_nullNo_falsePause_trueYes) {
                    recording_nullNo_falsePause_trueYes = false;
                }else {
                    recording_nullNo_falsePause_trueYes=null;
                }
                display.repaint();
            }
        }


        @Override
        public void mouseReleased(final MouseEvent e) {

            if (e.getButton() == MouseEvent.BUTTON1) {
                mouseIsDown = null;
            }
        }


        @Override
        public void mouseMoved(final MouseEvent e) {

            executeMotion(e);
        }


        @Override
        public void mouseDragged(final MouseEvent e) {

            executeMotion(e);
        }


        private void executeMotion(final MouseEvent e) {

            if (mouseIsDown == null) {
                return;
            }
            final Point currentLocation = e.getLocationOnScreen();
            final int deltaX = currentLocation.x - mouseIsDown.x;
            final int deltaY = currentLocation.y - mouseIsDown.y;
            mouseIsDown = currentLocation;

            final Point winPos = window.getLocation();
            window.setLocation(winPos.x + deltaX, winPos.y + deltaY);
        }

    }

}
